package com.usky.iot.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.agbox.RemotePatrolAgBoxService;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.iot.domain.BaseAppInfo;
import com.usky.iot.domain.BaseGgpFacility;
import com.usky.iot.mapper.BaseAppInfoMapper;
import com.usky.iot.service.BaseAppInfoService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.iot.service.config.rabbitmq.RabbitMQConfig;
import com.usky.iot.service.vo.AppInfoRequest;
import com.usky.iot.service.vo.BaseGgpFacilityRequest;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * APP用户心跳信息 服务实现类
 * </p>
 *
 * @author han
 * @since 2024-03-29
 */
@Service
public class BaseAppInfoServiceImpl extends AbstractCrudService<BaseAppInfoMapper, BaseAppInfo> implements BaseAppInfoService {

    @Autowired
    private RemotePatrolAgBoxService remotePatrolAgboxService;

    @Value("${agBox.push}")
    private Integer pushFlag;

    @Resource
    private RabbitTemplate rabbitTemplate;

    @Autowired
    private RabbitMQConfig rabbitMQConfig;

    public String getIpAddress(HttpServletRequest request) {
        String ip = request.getHeader("X-Forwarded-For");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

    public static HttpServletRequest getHttpServletRequest() {
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        if(requestAttributes == null){
            return null;
        }

        return ((ServletRequestAttributes) requestAttributes).getRequest();
    }

    @Override
    public void add(BaseAppInfo baseAppInfo){
//        JSONObject jsonObj = new JSONObject();
//        jsonObj.put("deviceId", baseAppInfo.getDeviceId());
//        baseAppInfo.setNetworkService(a.toJSONString());
        HttpServletRequest request = getHttpServletRequest();
        String accessIp = getIpAddress(request);
        baseAppInfo.setAccessIp(accessIp);
        baseAppInfo.setCreateBy(SecurityUtils.getUsername());
        baseAppInfo.setCreateTime(LocalDateTime.now());
        baseAppInfo.setTenantId(SecurityUtils.getTenantId());

        Map<String,Object> map = new HashMap<>();
        map.put("deviceId",baseAppInfo.getDeviceId());
        map.put("userName",baseAppInfo.getUserName());
        map.put("createTime",baseAppInfo.getCreateTime());
        rabbitTemplate.convertAndSend(rabbitMQConfig.patrolFInfoExchange,"",JSONObject.toJSONString(map));

        this.save(baseAppInfo);
//        if (pushFlag.equals(1)){
//            JSONObject a = remotePatrolAgboxService.updateHeart(jsonObj.toJSONString());
//        }
    }

    @Override
    public CommonPage<BaseAppInfo> page(AppInfoRequest appInfoRequest) {
        IPage<BaseAppInfo> page = new Page<>(appInfoRequest.getCurrent(), appInfoRequest.getSize());
        QueryWrapper<BaseAppInfo> queryWrapper = Wrappers.query();
        queryWrapper.select("id","device_id as deviceId","user_name as userName","device_model as deviceModel")
                .in("id","215");
        page = this.page(page,queryWrapper);
        return new CommonPage<>(page.getRecords(),page.getTotal(),page.getCurrent(),page.getSize());
    }
}
